PROGRAM atompaw
  !***************************************************************
  !   4-30-04 minor changes to simplify options
  !   2-29-04 major changes to code structure -- adding possibility
  !    of adjusting Vloc to norm conserving pseudopotential at given
  !      l
  !  4-19-00 pgm written by N. A. W. Holzwarth
  !     Calculates projector and basis functions needed by pwpaw pgm
  !       for electronic structure calculations using the PAW method
  !       of Blochl
  !     Modified verion of original genproj pgm
  !***************************************************************
  USE GlobalMath
  USE atomdata
  USE aeatom
  USE pseudo
  USE basis
  IMPLICIT NONE
  CHARACTER (len=4) :: flnm  
  CHARACTER (len=20) :: nm  
  CHARACTER (len=2) :: sym  
  CHARACTER (len=1) :: syml  
  REAL(8), POINTER :: r(:),den(:),rv(:),wfn(:,:)
  INTEGER, POINTER :: n,norbit,nps,npp,npd,npf,npg
  REAL(8), POINTER :: h
  INTEGER, PARAMETER :: VNCT=1,VNCK=2,VSHAPE=3
  CHARACTER(50) :: Vlocaltype
  INTEGER :: Vlocalindex
  INTEGER :: i,j,io,many,l,istart,irc,OK,nbase,ib,ic,icount
  INTEGER :: llmin, llmax, ll,lcount,jcount,id,ie,iop,lp
  REAL(8) :: rc1,rc,x,rr,eself,cc,q00,evale,storeself,qcore,Etotal
  REAL(8) :: tildekin,tildepot,onehat,onehartree,cexc,texc,vexc1
  REAL(8) :: vtexc,vtexc1,oneenergy,ekin,vlocal,tq,fac,stuff,term
  REAL(8), ALLOCATABLE :: ttphi(:),soij(:),stij(:),svij(:)
  REAL(8), ALLOCATABLE :: shij(:),snij(:),dum(:),rh(:),rth(:)
  REAL(8), ALLOCATABLE :: shartree(:),sshartree(:,:,:),wf(:),twf(:)
  REAL(8), ALLOCATABLE :: bm(:),cm(:),dm(:,:)
  INTEGER, PARAMETER :: ifen=11,ifatompaw=12,ifout=13
  INTEGER :: lcao_points,lcao_i,iskip
  REAL(8) :: tphirange,hlcao
  TYPE (GridInfo), TARGET :: AEGrid
  TYPE (PotentialInfo), TARGET :: AEPot
  TYPE (OrbitInfo), TARGET :: AEOrbit
  TYPE (OrbitInfo), TARGET :: PSOrbit
  TYPE (SCFInfo), TARGET :: AESCF
  TYPE (FCInfo), TARGET :: FC
  TYPE (PseudoInfo), TARGET :: PAW

  CALL Init_GlobalConstants
  CALL iSCFatom(AEGrid,AEPot,AEOrbit,AESCF)
  CALL ChooseValence(AEGrid,AEOrbit,FC)
  CALL FCenergy(AEGrid,AEPot,AEOrbit,AESCF,FC)

  OPEN(ifen,file=TRIM(AEPot%sym),form='formatted')

  WRITE(ifen,'("Atom = ",a2,"  Z = ",i4)')  AEPot%sym, AEPot%nz
  SELECT CASE(TRIM(exctype))
  CASE default
     WRITE(6,*) 'Perdew-Wang correlation'
     WRITE(ifen,*) 'Perdew-Wang correlation'
  CASE('LDA-PW')
     WRITE(6,*) 'Perdew-Wang correlation'
     WRITE(ifen,*) 'Perdew-Wang correlation'
  CASE('GGA-PBE')
     WRITE(6,*) 'Perdew - Burke - Ernzerhof GGA'
     WRITE(ifen,*) 'Perdew - Burke - Ernzerhof GGA'
  END SELECT

  n=>AEGrid%n
  h=>AEGrid%h
  r=>AEGrid%r
  rv=>AEPot%rv
  WRITE(ifen,'("Integration params -- n,rmax = ",i5,1p2e15.7)') &
       AEGrid%n,AEGrid%r(n)
  WRITE(ifen,*) ' all-electron results '
  WRITE(ifen,*) ' core states (zcore) = ',FC%zcore
  DO io=1,FC%norbit
     IF (FC%iscore(io))THEN
        WRITE(ifen,'(3i5,1p2e15.7)') io,AEOrbit%np(io),AEOrbit%l(io),&
             AEOrbit%occ(io),AEOrbit%eig(io)
     ENDIF
  ENDDO
  WRITE(ifen,*) ' valence states (zvale) = ',FC%zvale
  DO io=1,FC%norbit
     IF (.NOT.FC%iscore(io)) THEN
        WRITE(ifen,'(3i5,1p2e15.7)') io,AEOrbit%np(io),AEOrbit%l(io),&
             AEOrbit%occ(io),AEOrbit%eig(io)
     ENDIF
  ENDDO
  WRITE(ifen,*) 'evale = ', FC%evale
  !CALL flush(ifen)

  WRITE(6,*) 'Enter maximum L for basis and projector functions'
  READ(5,*) PAW%lmax
  PAW%rc=0
  WRITE(6,*) 'enter  rc '
  READ(5,*) rc1
  IF (rc1.LE.0.d0) THEN
     WRITE(6,*) 'error -- rc too small ',rc1
     STOP
  ENDIF
  irc=rc1/h
  ! -- define irc = even index gt irc -- need to moniter --
  irc=2*((irc+2)/2)+2 ; PAW%irc=irc
  rc=irc*h ; PAW%rc=rc
  WRITE(6,*) ' adjusted rc ',rc, r(irc)
  WRITE(6,*) 'irc,rc = ',irc,rc
  WRITE(ifen,*) ' paw parameters: '
  WRITE(ifen,*) '      lmax = ',PAW%lmax
  WRITE(ifen,*) '        rc = ',rc
  WRITE(ifen,*) '       irc = ',irc
  !
  CALL setbasis(AEGrid,AEPot,AEOrbit,FC,PAW)

  ALLOCATE(ttphi(n),dum(n),stat=OK)
  IF(OK /= 0) THEN
     WRITE(6,*) 'Error in allocating VNC ', n
     STOP
  ENDIF
  CALL initpseudopot(AEGrid,PAW)
  WRITE(6,*) ' This code uses SINC2 shape for projector '&
       ,' generation and for hat densities'
  CALL sethat(AEGrid,PAW)

  WRITE(6,*) ' Enter type of Vloc -- ',&
       'VNCT (Norm conserving - Troullier-Martins)',&
       'VNCK (Norm conserving - Kerker)',&
       ' or VSHAPE (for fixed shape form)'
  READ(5,*) Vlocaltype
  vlocalindex=VNCT
  IF (TRIM(Vlocaltype)=="VNCT") vlocalindex=VNCT
  IF (TRIM(Vlocaltype)=="VNCK") vlocalindex=VNCK
  IF (TRIM(Vlocaltype)=="VSHAPE") vlocalindex=VSHAPE

  IF (vlocalindex==VNCT) THEN
     CALL troullier(AEGrid,AEPot,PAW)
     CALL makebasis(AEGrid,PAW)
     CALL FindVlocfromVeff(AEGrid,FC,AEPot%nz,PAW)
  ENDIF

  IF (vlocalindex==VNCK) THEN
     CALL kerker(AEGrid,AEPot,PAW)
     CALL makebasis(AEGrid,PAW)
     CALL FindVlocfromVeff(AEGrid,FC,AEPot%nz,PAW)
  ENDIF

  IF (vlocalindex==VSHAPE) THEN
     CALL SCbasis(AEGrid,FC,AEPot%nz,PAW)
  ENDIF

  WRITE(ifen,*) PAW%Vloc_description

  CALL checkghosts(AEGrid,AEOrbit,FC,PAW)

  OPEN(ifout,file='density', form='formatted')
  DO i=1,n
     IF (FC%coreden(i)<machine_zero) FC%coreden(i)=0
     IF (PAW%den(i)<machine_zero) PAW%den(i)=0
     IF (PAW%tden(i)<machine_zero) PAW%tden(i)=0
     WRITE(ifout,'(1p1e15.7,1p3e25.17)') r(i),FC%coreden(i),&
          PAW%den(i),PAW%tden(i)
  ENDDO
  CLOSE(ifout)

  OPEN(ifout,file='potential', form='formatted')
  DO i=1,n
     IF (ABS(rv(i))<machine_zero) rv(i)=0
     IF (PAW%den(i)<machine_zero) PAW%rveff(i)=0
     WRITE(ifout,'(1p1e15.7,1p3e25.17)') r(i),AEPot%rv(i),PAW%rveff(i)
  ENDDO
  CLOSE(ifout)

  OPEN(ifout,file='vloc', form='formatted')
  DO i=1,irc+10
     WRITE(ifout,'(1p1e15.7,1p3e25.17)') r(i),PAW%vloc(i)
  ENDDO
  CLOSE(ifout)

  nbase=PAW%nbase
  WRITE(ifen,'(/"Number of basis functions ",i5)') nbase
  WRITE(ifen,*)'No.   n    l      Energy         Cp coeff         Occ'

  DO io=1,nbase
     WRITE(ifen,'(3i5,1p3e15.7)') io,PAW%np(io),PAW%l(io),PAW%eig(io),&
          PAW%ck(io),PAW%occ(io)
     CALL mkname(io,flnm)
     OPEN(ifout,file='wfn'//TRIM(flnm),form='formatted')
     WRITE(ifout,*) 'l=',PAW%l(io),'basis function with energy  ',PAW%eig(io)
       DO i=1,irc+50
          WRITE(ifout,'(1p5e12.4)') r(i),PAW%ophi(i,io),PAW%otphi(i,io),PAW%otp(i,io)
       ENDDO
       CLOSE(ifout)
    ENDDO

    !Begin  constructing output  for pwpaw pgm

    qcore=FC%zcore
    OPEN(ifatompaw,file=TRIM(AEpot%sym)//'.atomicdata',form='formatted')
    WRITE(ifatompaw,'("  ATOMTYPE     ",a2)') AEpot%sym
    WRITE(ifatompaw,'("  ATOMXCTYPE     ",a10)') exctype
    WRITE(ifatompaw,'("  ATOMIC_CHARGE    ",i5)') AEpot%nz
    WRITE(ifatompaw,'("  CORE_CHARGE    ",1pe20.13)') qcore
    WRITE(ifatompaw,'("  RC         ",1pe20.13)') PAW%rc
    WRITE(ifatompaw,'("  SHAPE_TYPE  ",a20)') 'sinc2'

    WRITE(ifatompaw,'("  BASIS_SIZE    ",i5)') PAW%nbase
    WRITE(ifatompaw,'("  ORBITALS      ")')
    WRITE(ifatompaw,'(20i4)') (PAW%l(ib),ib=1,nbase)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("  MESH_SIZE    ",i10)') PAW%irc+1
    WRITE(ifatompaw,'("  MESH_STEP    ",1pe20.13)') AEGrid%h

    tphirange=10.d0       ! tphirange=8.d0
    lcao_points=800       ! lcao_points=600
    lcao_i=tphirange/AEGrid%h
    iskip=MAX(1,lcao_i/lcao_points)
    lcao_i=iskip*(lcao_i/iskip)+iskip
    hlcao=AEGrid%h*iskip
    lcao_points=lcao_i/iskip + 1
    WRITE(ifatompaw,'("  LCAO_SIZE  ",i10)') lcao_points
    WRITE(ifatompaw,'("  LCAO_STEP   ",1pe20.13)') hlcao

    WRITE(ifatompaw,'("   CORE_DENSITY   ")')
    WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(FC%coreden(i),i=1,PAW%irc)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("   SHAPE_FUNC   ")')
    WRITE(ifatompaw,'(1p3e25.17)') 1.d0,(PAW%hatshape(i),i=1,PAW%irc)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("   VLOCFUN      ")')
    x=PAW%vloc(3)+3*(PAW%vloc(1)-PAW%vloc(2))
    WRITE(ifatompaw,'(1p3e25.17)') x,(PAW%vloc(i),i=1,PAW%irc)
    WRITE(ifatompaw,'("  END     ")')

    DO ib=1,nbase
       WRITE(ifatompaw,'("   TPROJECTOR",i4," #p(r), for p(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%otp(i,ib),i=1,PAW%irc)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO

    DO ib=1,nbase
       WRITE(ifatompaw,'("   PHI",i5," #phi(r), for phi(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%ophi(i,ib),i=1,PAW%irc)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO

    DO ib=1,nbase
       WRITE(ifatompaw,'("   TPHI",i5," #tphi(r), for tphi(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%otphi(i,ib),i=1,PAW%irc)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO


    DO ib=1,nbase
       ttphi=PAW%tphi(:,ib)
       CALL trunk(n,h,ttphi(1:n),6.d0,10.d0)
       WRITE(ifatompaw,'("   TPHI_LCAO",i4," #tphi0(r) for tphi0(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(ttphi(j),j=iskip,lcao_i,iskip)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO

    ! spherical matrix elements
    icount=(nbase*(nbase+1))/2
    ALLOCATE(soij(icount),stij(icount),svij(icount),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating dia matrix elements',icount,ok
       STOP
    ENDIF

    ! allocate PAW data set storage for spherical matrix elements
    ALLOCATE(PAW%tvij(nbase,nbase),PAW%vhatij(nbase,nbase),PAW%v0ij(nbase,nbase),&
         PAW%vhijkl(nbase,nbase,nbase,nbase),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating storage for matrix elements',nbase,ok
       STOP
    ENDIF
    PAW%tvij=0; PAW%vhatij=0; PAW%v0ij=0; PAW%vhijkl=0

    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          IF (PAW%l(ib)==PAW%l(ic)) THEN
             l=PAW%l(ib)
             icount=icount+1
             CALL dqij(AEGrid,PAW,ib,ic,soij(icount))
             CALL dtij(AEGrid,PAW,ib,ic,stij(icount))
             CALL dvij(AEGrid,PAW,FC,AEPot%nz,ib,ic,svij(icount))
             PAW%tvij(ib,ic)=stij(icount)+svij(icount)
             PAW%tvij(ic,ib)=stij(icount)+svij(icount)
          ENDIF
       ENDDO
    ENDDO

    WRITE(ifatompaw,'("   OVERLAP_SIZE    ",i10)') icount
    WRITE(ifatompaw,'("   OVERLAP_MATRIX  ")')
    WRITE(ifatompaw,'(1p3e25.17)') (soij(ic),ic=1,icount)
    WRITE(ifatompaw,'("  END     ")')
    WRITE(ifatompaw,'("   KINETIC_ENERGY_MATRIX  ")')
    WRITE(ifatompaw,'(1p3e25.17)') (stij(ic),ic=1,icount)
    WRITE(ifatompaw,'("  END     ")')
    WRITE(ifatompaw,'("   V_ION_MATRIX  ")')
    WRITE(ifatompaw,'(1p3e25.17)') (svij(ic),ic=1,icount)
    WRITE(ifatompaw,'("  END     ")')
    !
    !  angularly dependent matrix elements
    !
    icount=(nbase*(nbase+1))*(PAW%lmax+1)
    ALLOCATE(shij(icount),snij(icount),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating L matrix elements',icount,ok
       STOP
    ENDIF

    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             icount=icount+1
             ! write(6,*) 'ib,ic,ll',ib,ic,ll,icount
             DO i=1,irc
                rr=h*i
                dum(i)=(rr**ll)*(PAW%ophi(i,ib)*PAW%ophi(i,ic) &
                     -PAW%otphi(i,ib)*PAW%otphi(i,ic))
             ENDDO
             snij(icount)=overint(irc,h,dum)
             if (ll==0)  then
                PAW%v0ij(ib,ic)=snij(icount)
                PAW%v0ij(ic,ib)=snij(icount)
             endif
             CALL hatpotL(AEGrid,PAW,ll,dum)
             DO i=1,irc
                dum(i)=dum(i)*PAW%otphi(i,ib)*PAW%otphi(i,ic)
             ENDDO
             shij(icount)=overint(irc,h,dum)
             if (ll==0)  then
                PAW%vhatij(ib,ic)=shij(icount)
                PAW%vhatij(ic,ib)=shij(icount)
             endif
          ENDDO
       ENDDO
    ENDDO

    WRITE(ifatompaw,'("   DENVHAT_SIZE   ",i10)') icount
    WRITE(ifatompaw,'("   DENSITY   ")')
    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             icount=icount+1
             WRITE(ifatompaw,'(3i10,1pe25.17)') ib,ic,ll,snij(icount)
          ENDDO
       ENDDO
    ENDDO
    WRITE(ifatompaw,'("   END   ")')
    WRITE(ifatompaw,'("   V_HAT   ")')
    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             icount=icount+1
             WRITE(ifatompaw,'(3i10,1pe25.17)') ib,ic,ll,shij(icount)
          ENDDO
       ENDDO
    ENDDO
    WRITE(ifatompaw,'("   END   ")')
    !
    icount=(nbase*(nbase+1))/2
    lcount=(icount**2)*((PAW%lmax+1)*2)
    ALLOCATE(shartree(lcount),sshartree(icount,icount,2*PAW%lmax+2),&
         wf(n),twf(n),rh(n),rth(n),stat=ok)
    IF (ok/=0) THEN
       WRITE(6,*) 'Error in hartree allocation', icount,lcount,irc,ok
       STOP
    ENDIF
    !
    !  Hartree matrix elements
    !
    !! Due to precision error in apoisson solver, some equivalent Hartree
    !!  matrix elements are not equal with an error of e-5; take average
    !!  these elements in order to remove assymmetry errors
    !!  Thanks to Francois Jollet for pointing out this problem
    sshartree=0
    shartree=0
    icount=0
    lcount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          icount=icount+1
          DO i=1,irc
             wf(i)=PAW%ophi(i,ib)*PAW%ophi(i,ic)
             twf(i)=PAW%otphi(i,ib)*PAW%otphi(i,ic)
          ENDDO
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             CALL apoisson(ll,irc,h,wf,rh)
             CALL apoisson(ll,irc,h,twf,rth)
             jcount=0
             DO id=1,nbase
                DO ie=id,nbase
                   jcount=jcount+1
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      !WRITE(6,*) 'ib,ic,ll',ib,ic,id,ie,ll,lcount,icount,jcount
                      DO i=1,irc
                         rr=h*i
                         dum(i)=PAW%ophi(i,id)*PAW%ophi(i,ie)*rh(i)/rr            &
                              -PAW%otphi(i,id)*PAW%otphi(i,ie)*rth(i)/rr
                      ENDDO
                      shartree(lcount)=overint(irc,h,dum(1:irc))
                      sshartree(icount,jcount,ll+1)=shartree(lcount)
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO

    icount=0
    lcount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          icount=icount+1
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             jcount=0
             DO id=1,nbase
                DO ie=id,nbase
                   jcount=jcount+1
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      shartree(lcount)=                      &
                           0.5d0*(sshartree(icount,jcount,ll+1) + &
                           sshartree(jcount,icount,ll+1))
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO

    WRITE(6,*) lcount,'   ; # Hartree matrix elements'
    WRITE(ifatompaw,'("   HARTREE_SIZE   ",i10)') lcount
    WRITE(ifatompaw,'("   V_HARTREE   ")')
    lcount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             DO id=1,nbase
                DO ie=id,nbase
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      WRITE(ifatompaw,'(5i5,1pe25.17)')ib,ic,id,ie,ll,shartree(lcount)
                      IF (ll==0) THEN
                        PAW%vhijkl(ib,ic,id,ie)=shartree(lcount)
                        PAW%vhijkl(ic,ib,id,ie)=shartree(lcount)
                        PAW%vhijkl(ib,ic,ie,id)=shartree(lcount)
                        PAW%vhijkl(ic,ib,ie,id)=shartree(lcount)
                        PAW%vhijkl(id,ie,ib,ic)=shartree(lcount)
                        PAW%vhijkl(id,ie,ic,ib)=shartree(lcount)
                        PAW%vhijkl(ie,id,ib,ic)=shartree(lcount)
                        PAW%vhijkl(ie,id,ic,ib)=shartree(lcount)
                      ENDIF
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    WRITE(ifatompaw,'("   END         ")')

    WRITE(ifatompaw,'("   HAT_SELF-ENERGY    ",i10)') 2*PAW%lmax
    DO ll=0,2*PAW%lmax
       CALL selfhatpot(AEGrid,PAW,ll,eself)
       WRITE(ifatompaw,'(i10,1pe25.17)') ll,eself
       IF (ll==0) storeself=eself
    ENDDO
    WRITE(ifatompaw,'("   END   ")')

    lcount=(nbase*(nbase+1))/2
    ALLOCATE(bm(nbase),cm(nbase),dm(lcount,lcount),stat=ok)
    IF (ok/=0) THEN
       WRITE(6,*) 'Error in allocation before evale', nbase,ok
       STOP
    ENDIF

    !
    !  using matrix elements to re-evaluate evale --
    !
    !
    tildekin=0.d0
    PAW%den=0; PAW%tden=0
    DO ib=1,nbase
       l=PAW%l(ib)
       CALL kinetic(AEGrid,PAW%l(ib),PAW%tphi(:,ib),PAW%tphi(:,ib),ekin)
       tildekin=tildekin+PAW%occ(ib)*ekin
       PAW%den=PAW%den+PAW%occ(ib)*(PAW%phi(:,ib))**2
       PAW%tden=PAW%tden+PAW%occ(ib)*(PAW%tphi(:,ib))**2
    ENDDO

    DO i=1,irc
       dum(i)=PAW%den(i)-PAW%tden(i)
    ENDDO
    q00=-AEPot%nz+FC%zcore+overint(irc,h,dum)
    WRITE(6,*) 'q00 = ',q00
    DO i=1,n
       dum(i)=(PAW%tden(i)+q00*PAW%hatden(i))
    ENDDO
    tq=overint(n,h,dum)
    WRITE(6,*) 'tq  = ',tq
    CALL poisson(n,h,tq,dum,ttphi,x)
    tildepot=x+overlap(n,h,PAW%vloc,PAW%tden)

    oneenergy=0.d0
    DO io=1,nbase
       l=PAW%l(io)
       bm=0.d0
       DO ib=1,nbase
          IF (PAW%l(ib).EQ.l)  THEN
             bm(ib)=overlap(n,h,PAW%otp(:,ib),PAW%tphi(:,io))
             IF (io==ib) THEN
                IF (ABS(bm(ib)-1).GT.1.d-11)&
                     WRITE(6,*) 'Warning in pdot',io,ib,bm(ib)
             ELSE
                IF (ABS(bm(ib)).GT.1.d-11)&
                     WRITE(6,*) 'Warning in pdot',io,ib,bm(ib)
             ENDIF
          ENDIF
       ENDDO
       lcount=0
       DO ib=1,nbase
          DO ic=ib,nbase
             IF (PAW%l(ib).EQ.PAW%l(ic)) THEN
                lcount=lcount+1
                fac=PAW%occ(io)
                IF(ic.GT.ib) fac=2*fac
                oneenergy=oneenergy+                                         &
                     fac*bm(ib)*bm(ic)*(stij(lcount)+svij(lcount))
             ENDIF
          ENDDO
       ENDDO
    ENDDO

    onehat=0.d0
    DO io=1,nbase
       l=PAW%l(io)
       bm=0.d0
       DO ib=1,nbase
          IF (PAW%l(ib).EQ.l)                                       &
               bm(ib)=overlap(n,h,PAW%otp(:,ib),PAW%tphi(:,io))
       ENDDO
       lcount=0
       DO ib=1,nbase
          DO ic=ib,nbase
             llmin=ABS(PAW%l(ib)-PAW%l(ic))
             llmax=PAW%l(ib)+PAW%l(ic)
             fac=PAW%occ(io)
             IF (ic.GT.ib) fac=2*fac
             DO ll=llmin,llmax,2
                lcount=lcount+1
                IF (ll.EQ.0) THEN
                   onehat=onehat+q00*fac*bm(ib)*bm(ic)*shij(lcount)
                ENDIF
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    onehat=onehat + storeself*(q00**2)

    onehartree=0.d0
    ! load array dm
    lcount=0
    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          icount=icount+1
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             jcount=0
             DO id=1,nbase
                DO ie=id,nbase
                   jcount=jcount+1
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      IF (ll.EQ.0) dm(icount,jcount)=shartree(lcount)

                   ENDIF
                ENDDO !ie
             ENDDO   !id
          ENDDO    !ll
       ENDDO ! ic
    ENDDO  ! ib

    DO io=1,nbase
       l=PAW%l(io)
       bm=0.d0
       DO ib=1,nbase
          IF (PAW%l(ib).EQ.l)                                       &
               bm(ib)=overlap(irc,h,PAW%otp(:,ib),PAW%tphi(:,io))
       ENDDO
       icount=0
       DO ib=1,nbase
          DO ic=ib,nbase
             icount=icount+1
             stuff=PAW%occ(io)*bm(ic)*bm(ib)
             IF (ic.GT.ib) stuff=2*stuff
             term=0.d0
             DO iop=1,nbase
                lp=PAW%l(iop)
                cm=0.d0
                DO id=1,nbase
                   IF (PAW%l(id).EQ.lp)                                      &
                        cm(id)=overlap(irc,h,PAW%otp(:,id),PAW%tphi(:,iop))
                ENDDO
                jcount=0
                DO id=1,nbase
                   DO ie=id,nbase
                      jcount=jcount+1
                      fac=PAW%occ(iop)*cm(id)*cm(ie)
                      IF (ie.GT.id) fac=2*fac
                      term=term+fac*dm(icount,jcount)
                   ENDDO !ie
                ENDDO   !id
             ENDDO  !iop
             onehartree=onehartree+stuff*term
          ENDDO ! ic
       ENDDO  ! ib
    ENDDO  !io

    !  exchange-correlation energy
    !
    CALL exch(n,h,PAW%tden,dum,stuff,vtexc)
    CALL exch(irc,h,PAW%tden,dum,stuff,vtexc1)
    CALL exch(n,h,FC%coreden,dum,stuff,cc)
    CALL exch(irc,h,FC%coreden,dum,stuff,cexc)
    cc=cc-cexc
    DO i=1,irc
       wf(i)=FC%coreden(i)+PAW%den(i)
    ENDDO
    CALL exch(irc,h,wf,dum,stuff,vexc1)

    texc=vtexc+vexc1-vtexc1-cexc
    WRITE(6,*) ' exchange-correlation energy', texc
    WRITE(6,*) ' core tail exchange error ', cc
    !
    !  local potential contribution -- already in matrix element -- not added
    !
    vlocal=overlap(irc,h,PAW%vloc(1:irc),PAW%tden(1:irc))
    WRITE(6,*) 'vlocal contribution ', vlocal

    evale=tildekin+tildepot+oneenergy-onehat                        &
         +0.5d0*onehartree+texc
    WRITE (6,*) ' evale from matrix elements ', evale
    WRITE(6,*) ' evale from AE calculation ',FC%evale
    WRITE(6,*) ' difference in evale results ', evale-FC%evale
    WRITE(6,*) 'tildekin ', tildekin
    WRITE(6,*) 'tildepot ' , tildepot
    WRITE(6,*) 'oneenergy ', oneenergy
    WRITE(6,*) 'onehat ', onehat
    WRITE(6,*) 'onehartree ', onehartree
    WRITE(6,*) 'texc ',texc
    WRITE(6,*) 'vlocal contribution ', vlocal
    !
    WRITE(ifen,'("evale from matrix elements",1pe25.17)') evale
    WRITE(6,'("evale from matrix elements",1pe25.17)') evale

    WRITE(ifatompaw,'("   ENERGY   ",1pe25.17)') evale
    WRITE(ifatompaw,'("   END   ")')

    CALL ftprod(AEGrid,PAW)
    CALL logderiv(AEGrid,AEPot,PAW)
    CALL fthatpot(AEGrid,PAW)
    CALL ftkin(AEGrid,PAW)
    CALL ftvloc(AEGrid,PAW)
    !
    Do
       WRITE(6,*) 'Enter 0 to end program'
       WRITE(6,*) 'Enter 1 to run SCFPAW'
       READ(5,*) i
       if (i==0) exit
       CALL SCFPAW(AEGrid,PAW,FC,AEOrbit,PSOrbit,Etotal)      
  WRITE(ifen,*) ' '
  WRITE(ifen,*) ' PAW results for new configuration'
  WRITE(6,*) ' '
  WRITE(6,*) ' PAW results for new configuration'
  DO io=1,PSOrbit%norbit
        WRITE(ifen,'(3i5,1p2e15.7)') io,PSOrbit%np(io),PSOrbit%l(io),&
             PSOrbit%occ(io),PSOrbit%eig(io)
        WRITE(6,'(3i5,1p2e15.7)') io,PSOrbit%np(io),PSOrbit%l(io),&
             PSOrbit%occ(io),PSOrbit%eig(io)
  ENDDO
  WRITE(ifen,'("evale from matrix elements",1pe25.17)') Etotal
  WRITE(6,'("evale from matrix elements",1pe25.17)') Etotal
  !CALL flush(ifen)
    Enddo
  END PROGRAM atompaw
